import { spawn, spawnSync } from 'child_process';

export const checkPnpm = () => {
  try {
    spawnSync('pnpm', ['--version']);
    return true;
  } catch (e) {
    return false;
  }
};
export const checkPm2 = () => {
  try {
    spawnSync('pm2', ['--version']);
    return true;
  } catch (e) {
    return false;
  }
};

type InstallDepOptions = {
  appPath?: string;
  dep: string;
  isGlobal?: boolean;
  sync?: boolean;
};
export const installDep = (opts: InstallDepOptions) => {
  const { appPath, dep } = opts;
  const params = [];
  const syncSpawn = opts.sync ? spawnSync : spawn;
  if (opts.isGlobal) {
    params.push('-g');
  }
  if (checkPnpm()) {
    params.push('add', dep);
    syncSpawn('pnpm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  } else {
    params.push('install', dep);
    syncSpawn('npm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  }
};
type InstallDepsOptions = {
  appPath: string;
  isProduction?: boolean;
  sync?: boolean;
};
export const installDeps = (opts: InstallDepsOptions) => {
  const { appPath } = opts;
  const isProduction = opts.isProduction ?? true;
  const params = ['i'];
  if (isProduction) {
    params.push('--production');
  }
  const syncSpawn = opts.sync ? spawnSync : spawn;
  if (checkPnpm()) {
    syncSpawn('pnpm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  } else {
    syncSpawn('npm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  }
};
